library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;

LIBRARY LR;
USE LR.PackageLR.all;
USE LR.LancerRayon;

ENTITY ASICLR_Comp is 
	PORT (
		DataIn 	: IN BitVector32 ;
		DataOut	: OUT BitVector32 ;
		Adresses : IN BitVector3 ;
		cs 	: IN bit ;
		rw 	: IN bit ;
		clock 	: IN bit ;
		reset 	: IN bit) ;
END ASICLR_Comp ;

ARCHITECTURE structure OF ASICLR_Comp IS

	COMPONENT lancer_de_rayon 
	PORT (	Px, Py, Pz 	: IN INTEGER;	
		Dx, Dy, Dz 	: IN INTEGER;	
		Vx, Vy, Vz 	: IN INTEGER;	
		Rcarre 		: IN INTEGER;
		lambda 		: OUT INTEGER;	
		debut 		: IN bit);
	END COMPONENT ;

	SIGNAL Signal_Px : INTEGER := 0;
	SIGNAL Signal_Py : INTEGER := 0;
	SIGNAL Signal_Pz : INTEGER := 0;
	SIGNAL Signal_Dx : INTEGER := 0;
	SIGNAL Signal_Dy : INTEGER := 0;
	SIGNAL Signal_Dz : INTEGER := 0;
	SIGNAL Signal_Vx : INTEGER := 0;
	SIGNAL Signal_Vy : INTEGER := 0;
	SIGNAL Signal_Vz : INTEGER := 0;

	SIGNAL S_Rcarre  : INTEGER := 0;
	SIGNAL S_Debut   : bit := '0';
	SIGNAL S_lambda  : INTEGER := 0;

	for lr0 : lancer_de_rayon  
		USE entity LR.LancerRayon(comportement);
 	
BEGIN

	-- Ce processus transforme un acces bus en signaux Pxyz, Dxyz, Vxyz, R, Lambda

	Interface : PROCESS (cs, rw, clock)
		VARIABLE ad : INTEGER;
	BEGIN
		IF cs = '0' AND clock = '1' AND clock'EVENT THEN 
			ad := CONV_INTEGER(Adresses);
			IF (ad = 0) AND (rw = '0')  THEN 
				Signal_Px <= CONV_INTEGER(DataIn(31 DOWNTO 24));
				Signal_Py <= CONV_INTEGER(DataIn(23 DOWNTO 16));
				Signal_Pz <= CONV_INTEGER(DataIn(15 DOWNTO 8));
				S_Debut <= '1';
			END IF;
			IF (ad = 1) AND (rw = '0') THEN 
				Signal_Dx <= CONV_INTEGER(DataIn(31 DOWNTO 24));
				Signal_Dy <= CONV_INTEGER(DataIn(23 DOWNTO 16));
				Signal_Dz <= CONV_INTEGER(DataIn(15 DOWNTO 8));
				S_Debut <= '1';
			END IF;
			IF (ad = 2) AND (rw = '0') THEN 
				Signal_Vx <= CONV_INTEGER(DataIn(31 DOWNTO 24));
				Signal_Vy <= CONV_INTEGER(DataIn(23 DOWNTO 16));
				Signal_Vz <= CONV_INTEGER(DataIn(15 DOWNTO 8));
				S_Rcarre  <= CONV_INTEGER(DataIn(7 DOWNTO 0));
				S_Debut <= '1';
			END IF;
			IF ad = 3 AND rw = '1' THEN 
				DataOut(31 DOWNTO 16) <= CONV_SIGNED(S_lambda,16) ;
 				S_Debut <= '0';
			END IF;
		END IF;
	END PROCESS ;

	lr0 : lancer_de_rayon PORT MAP (
			Signal_Px, 
			Signal_Py, 
			Signal_Pz, 

			Signal_Dx, 
			Signal_Dy, 
			Signal_Dz, 

			Signal_Vx, 
			Signal_Vy, 
			Signal_Vz, 

			S_Rcarre, 

			S_lambda,
			S_Debut);

END structure;



